import java.io.*;
import java.sql.*;
import se.datadosen.jalbum.AlbumBean;
import se.datadosen.jalbum.AlbumImage;
import java.util.Hashtable;
import javax.imageio.*;
import java.awt.image.*;
import java.awt.Image;
import java.awt.*;
import javax.imageio.stream.*;
import java.util.*;

public class GalleryGenerator
{
	private int userId;
	private String user;
	private int visibility;
	private String startDir;
	private char[] forbiddenChars;
	private String thumbPrefix;
	private String normPrefix;
	private Hashtable properties;
	private Hashtable fileTypes;
	private boolean recreateDerivatives = false;
	private boolean useShell = true;
	private String copperMineAlbumHome;

	public static void main(String[] args)
	{
		if(args.length!=7)
		{
			debug("usage copperMineAlbumHomeDir startdir user visibility dbUser dbPassword dbUrl(example: jdbc:mysql://localhost/cpg133) [recreateDerivatives]");
			System.exit(-1);
		}
		GCJDBCAdapter.getInstance(args[4], args[5],args[6]);
		GalleryGenerator generator = new GalleryGenerator(args[0], args[1], args[2], Integer.parseInt(args[3]));
		if(args.length >= 8 && args[7].equals("recreateDerivatives"))
		{
			generator.recreateDerivatives = true;
		}
		ImageIO.scanForPlugins();
		generator.traverse();
	}
	
	public GalleryGenerator(String copperMineAlbumHome, String startDir, String user, int visibility)
	{

		this.user=user;
		this.startDir=startDir;
		this.visibility=visibility;
		this.copperMineAlbumHome = copperMineAlbumHome;
		userId=0;
		loadUserId();
		loadProperties();
		loadForbiddenChars();
		loadThumbPrefix();
		loadNormPrefix();
		loadFileTypes();

	}
	private void loadThumbPrefix()
	{
		thumbPrefix=(String)properties.get("thumb_pfx");
		if (thumbPrefix == null )
		{
			debug("No thumbPrefix found");
			System.exit(-5);
		}
	}
	private void loadNormPrefix()
	{
		normPrefix=(String)properties.get("normal_pfx");
		if (normPrefix == null )
		{
			debug("No normPrefix found");
			System.exit(-5);
		}
	}
	private void loadForbiddenChars()
	{	
		String forbiddenChars=(String)properties.get("forbiden_fname_char");
		if (forbiddenChars == null )
		{
			debug("No forbidden chars found");
			System.exit(-5);
		}
		
		debug("Forbidden chars before: " + forbiddenChars);
		forbiddenChars = forbiddenChars.replaceAll("&amp;","&");
		forbiddenChars = forbiddenChars.replaceAll("&quot;","\"");
		forbiddenChars = forbiddenChars.replaceAll("&lt;","<");
		forbiddenChars = forbiddenChars.replaceAll("&gt;",">");
		debug("Forbidden chars after: " + forbiddenChars);
		this.forbiddenChars = forbiddenChars.toCharArray();
	}

	private void loadFileTypes()
	{
		fileTypes = new Hashtable(100);
		PreparedStatement stmt  =  null;

		ResultSet rs = null;
		String SQL = "SELECT extension,content FROM cpg133_filetypes";
		try
		{
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			
			rs = stmt.executeQuery();
			while(rs.next())
			{
				fileTypes.put(rs.getString(1), rs.getString(2));
			}
			
			rs.close();
			stmt.close();
			
			
		} catch(SQLException sqe)
		{
			sqe.printStackTrace();
		}
		finally
		{
			try{ 
				if(rs != null)
				rs.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
			try{ 
				if(stmt != null)
				stmt.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
		}		
	}

	private void loadProperties()
	{
		properties = new Hashtable(100);
		PreparedStatement stmt  =  null;

		ResultSet rs = null;
		String SQL = "SELECT name,value FROM cpg133_config";
		try
		{
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			
			rs = stmt.executeQuery();
			while(rs.next())
			{
				properties.put(rs.getString(1), rs.getString(2));
			}
			
			rs.close();
			stmt.close();
			
			
		} catch(SQLException sqe)
		{
			sqe.printStackTrace();
		}
		finally
		{
			try{ 
				if(rs != null)
				rs.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
			try{ 
				if(stmt != null)
				stmt.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
		}		
	}
	private void loadUserId()
	{
		PreparedStatement stmt  =  null;
		
		ResultSet rs = null;
		String SQL = "SELECT user_id FROM cpg133_users WHERE user_name like ?";
		try
		{
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			stmt.setString(1,user);
			
			rs = stmt.executeQuery();
			if(rs.next())
			{
				userId=rs.getInt(1);
			}
			else
			{	
				debug("No user found!");
				System.exit(-3);
			}
			rs.close();
			stmt.close();

		} catch(SQLException sqe)
		{
			sqe.printStackTrace();
		}
		finally
		{
			try{ 
				if(rs != null)
				rs.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
			try{ 
				if(stmt != null)
				stmt.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
		}

	}
	public void traverse()
	{
		traverse(this.startDir,0);
	}
	private void traverse(String startDir, int parentCategoryId)
	{
		debug("Traversing " + startDir);
		File startFolder = new File(startDir);
		boolean hasSubFolders = false;
		boolean hasFiles = false;
		int categoryId = -1;
		int albumId = -1;
	
		if(startFolder.exists() == false || startFolder.isDirectory() == false)
		{
			debug("Startdir " + startDir + " does not exist!");
			System.exit(-2);
		}
		File[] children = startFolder.listFiles();
		
		//go through all of them to see if there might be a file somewhere
		for(int i = 0; i < children.length; i++)
		{
			if(children[i].isDirectory())
			{
				hasSubFolders = true;
				if(categoryId < 0)
				{
				 	debug("Call createCategory() here! Return Category Id! " + children[i]);	
					categoryId = createCategory(startFolder.getName(), parentCategoryId);
					//categoryId = 1;
				}
				traverse(children[i].getAbsolutePath(),categoryId);
			}
			else
			{
				hasFiles = true;
				if(albumId  < 0)
				{
					debug("Call createAlbum() here! Return Album Id! " + children[i]);	
					albumId  = createAlbum(startFolder.getName(), parentCategoryId);
				}

			}
		}
		if(children.length==0)
		{
			//create an empty category...	
			categoryId = createCategory(startFolder.getName(), parentCategoryId);
		}
		//now we looked at the files once, let's add files that are not folders...
		if(hasFiles)
		{

			for(int i =0; i < children.length; i++)
			{
				if(!children[i].isDirectory())
				{	
					debug("Trying to add "  + children[i].getName() + " to " + albumId);
					try{
						addImage(albumId, startFolder, children[i]);
					}
					catch(IOException ioe)
					{	
						ioe.printStackTrace();	
						System.exit(-11);
					}
				}
			}
		}
		
		

	}

	private int createCategory(String categoryName , int parentCategoryId)
	{
		PreparedStatement stmt  =  null;
		int categoryId = 0;
		ResultSet rs = null;

		//first check if it already exists...
		String SQL = "SELECT cid FROM cpg133_categories where name like ? and parent=?";
		
		 
		try
		{
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			stmt.setString(1,categoryName);
			stmt.setInt(2,parentCategoryId);
			rs = stmt.executeQuery();


			if(rs.next())
			{
				categoryId = rs.getInt(1);
				debug("This category already exists...");
				rs.close();
				stmt.close();
				return categoryId;
			}
			rs.close();
			stmt.close();

			SQL = "insert into cpg133_categories (name, owner_id, pos, parent) values";
			SQL =  SQL + " (?,?,?,?)";	
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			stmt.setString(1,categoryName);
			stmt.setInt(2,userId);
			stmt.setInt(3,0);
			stmt.setInt(4,parentCategoryId);
			stmt.executeUpdate();

			stmt.close();

			SQL = "SELECT cid FROM cpg133_categories WHERE cid IS NULL";
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			rs = stmt.executeQuery();


			if(rs.next())
			{
				categoryId= rs.getInt(1);
			}
			rs.close();
			stmt.close();
			return categoryId;	
		} catch(SQLException sqe)
		{
			sqe.printStackTrace();
		}
		finally
		{
			try{ 
				if(rs != null)
				rs.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
			try{ 
				if(stmt != null)
				stmt.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
		}
		return -1;	
	}

	private int createAlbum(String albumName, int parentCategoryId)
	{
		PreparedStatement stmt  =  null;
		ResultSet rs = null;
		int albumId=0;
		//first check if it already exists...
		String SQL = "SELECT aid FROM cpg133_albums where title like ? and category=?";
		
		 
		try
		{
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			stmt.setString(1,albumName);
			stmt.setInt(2,parentCategoryId);
			rs = stmt.executeQuery();


			if(rs.next())
			{
				albumId = rs.getInt(1);
				debug("This Album already exists...");
				rs.close();
				stmt.close();
				return albumId;
			}

			rs.close();
			stmt.close();	
			SQL = "insert into cpg133_albums (title, pos, visibility,category) values";
			SQL =  SQL + " (?,?,?,?)";
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			stmt.setString(1,albumName);
			stmt.setInt(2,0);
			stmt.setInt(3,visibility);
			stmt.setInt(4,parentCategoryId);
			stmt.executeUpdate();

			stmt.close();

			SQL = "SELECT aid FROM cpg133_albums WHERE aid IS NULL";
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			rs = stmt.executeQuery();


			if(rs.next())
			{
				albumId= rs.getInt(1);
			}
			rs.close();
			stmt.close();
			return albumId;	
		} catch(SQLException sqe)
		{
			sqe.printStackTrace();

		}
		finally
		{
			try{ 
				if(rs != null)
				rs.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
			try{ 
				if(stmt != null)
				stmt.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
		}	
		return -1;
	}
	private int addImage(int albumId, File album, File myImage) throws IOException
	{


			String myNewImageName =  myImage.getName();
			int imageId=-1;
			int thumbSize =0;
			int previewSize=0;
			BufferedImage myImageHandler=null;

			//check if norm or thumb...
			if(myNewImageName.indexOf(thumbPrefix) == 0)
			{
				debug(myNewImageName + " is a thumb...");
				return -1;
			}
			else if(myNewImageName.indexOf(normPrefix) == 0)
			{
				debug(myNewImageName + " is a norm...");
				return -2;
			}
	
			//Get forbidden chars from database and rename...
			//translate from escape chars... '&amp;' => '&', '&quot;' => '"', '&lt;' => '<', '&gt;' => '>'
			for(int i=0; i < this.forbiddenChars.length;i++)
			{
				myNewImageName=myNewImageName.replace(this.forbiddenChars[i], '_');
			}
			if(!myImage.renameTo(new File(album, myNewImageName)))
			{
				debug("Did not succeed in renaming " + myImage + " to " + myNewImageName);
				System.exit(-4);
			}
			else 
			{
				debug("Renamed " + myImage + " to " + myNewImageName);
			}	

			//check if already imported...
			imageId = alreadyImported(albumId, myImage);
			debug("Check if allowed to add "  + myImage.getName() + " to " + albumId + " result=" + imageId);

			if(imageId != -1)
			{
				System.out.println("Image " + myImage + " already in DB...");
				if(recreateDerivatives && isImage(myImage.getName()))
				{	
					myImageHandler = ImageIO.read(myImage);
					if(myImage == null)
					{
						throw new IOException("Couldn't read image" + myImage.getPath());	
					}
					thumbSize = createThumbnail(albumId,myImage, myImageHandler);
					previewSize = createPreview(albumId,myImage, myImageHandler);
				}

			}
			else if(isKnownFileType(myImage.getName()))
			{
				debug("Ok to add "  + myImage.getName() + " to " + albumId + " (" + album.getName() + ")");
				if(isImage(myImage.getName()))
				{
					myImageHandler = ImageIO.read(myImage);
					if(myImage == null)
					{
						throw new IOException("Couldn't read image" + myImage.getPath());	
					}
					thumbSize = createThumbnail(albumId,myImage, myImageHandler);
					previewSize = createPreview(albumId,myImage, myImageHandler);
				}
				int totalSize = thumbSize + previewSize + (int) myImage.length();
				createDatabaseEntry(albumId,myImage, myImageHandler,totalSize);
			}
			return imageId;				
	}

	private int alreadyImported(int albumId,File myImage)
	{
		PreparedStatement stmt  =  null;
		ResultSet rs = null;
		int retval = -1;
		String SQL = "SELECT pid from cpg133_pictures WHERE aid=? AND filename like ? LIMIT 1";
		try
		{
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			stmt.setInt(1,albumId);
			stmt.setString(2,myImage.getName());
			
			rs = stmt.executeQuery();
			if(rs != null && rs.next())
			{
				retval = rs.getInt(1);
			}
			else
			{	
				retval = -1;
			}

		} catch(SQLException sqe)
		{
			sqe.printStackTrace();
		}
		finally
		{
			try{ 
				if(rs != null)
				rs.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
			try{ 
				if(stmt != null)
				stmt.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
		}
		return retval;
	}

	private int createDatabaseEntry(int albumId,File myImageFile, BufferedImage myImage, int totalSize)
	{
		PreparedStatement stmt  =  null;
		ResultSet rs = null;
		int retval = -1;
		String basePath = (new File(this.copperMineAlbumHome)).getPath();

		System.out.println("Base Path is: " + basePath);
		String filePath = myImageFile.getPath();

		int relativeStarts = filePath.indexOf(basePath)+ basePath.length()+1;

		String relativePath = (myImageFile.getParent()).substring(relativeStarts) +"/";
		relativePath = relativePath.replace('\\' , '/');
		System.out.println("Relative Path is: " + relativePath);
		String fileName = myImageFile.getName();
		int width = myImage != null ? myImage.getWidth() : 0 ;
		int height = myImage != null ? myImage.getHeight(): 0 ;

		String SQL = "INSERT INTO cpg133_pictures (aid, filepath, fileName, filesize,total_filesize,pwidth, pheight,owner_id, owner_name )";
		SQL = SQL + " values (?,?,?,?,?,?,?,?,?)";
		try
		{
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			stmt.setInt(1,albumId);
			stmt.setString(2,relativePath);
			stmt.setString(3,fileName);
			stmt.setLong(4,myImageFile.length());
			stmt.setInt(5,totalSize);
			stmt.setInt(6,width);
			stmt.setInt(7,height);
			stmt.setInt(8,userId);
			stmt.setString(9,user);
			
			stmt.executeUpdate();
			stmt.close();

			SQL = "SELECT pid FROM cpg133_pictures WHERE pid IS NULL";
			stmt = GCJDBCAdapter.getInstance().getPreparedStatement(SQL);
			rs = stmt.executeQuery();

			if(rs != null && rs.next())
			{
				retval = rs.getInt(1);
			}
			else
			{	
				retval = -1;
			}
			return retval;
		} catch(SQLException sqe)
		{
			sqe.printStackTrace();
		}
		finally
		{
			try{ 
				if(rs != null)
				rs.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
			try{ 
				if(stmt != null)
				stmt.close();
			}catch(SQLException sqe)
			{	
				sqe.printStackTrace();
			}
		}
		return retval;
	}
	
	private int createThumbnail(int albumId,File myImageFile, BufferedImage myImage) throws IOException
	{
		int thumbWidth = Integer.parseInt(((String)properties.get("thumb_width")));
		String thumbMethod = (String)properties.get("thumb_method");
		String thumbUse = (String)properties.get("thumb_use");
		File thumbFile = new File(myImageFile.getParent(),  thumbPrefix +  myImageFile.getName());
		debug("Writing thumbnail to " + thumbFile.getPath());
		return resizeImage(myImageFile, thumbFile, thumbWidth, thumbMethod, thumbUse, myImage);
			
		
	}
	private int createPreview(int albumId,File myImageFile, BufferedImage myImage) throws IOException
	{
		int previewWidth = Integer.parseInt(((String)properties.get("picture_width")));
		String previewMethod = (String)properties.get("thumb_method");
		String previewUse = (String)properties.get("thumb_use");
		File previewFile = new File(myImageFile.getParent(),  normPrefix +  myImageFile.getName());
		if(Math.max(myImage.getWidth(),myImage.getHeight()) > previewWidth && (properties.get("make_intermediate")).equals("1"))	
		{	
			debug("Writing preview to " + previewFile.getPath());
			return resizeImage(myImageFile, previewFile, previewWidth, previewMethod, previewUse, myImage);
		}
		return 0;
	}

	private int resizeImage(File sourceFile, File destinationFile,int newSize, String thumbMethod, String thumbUse, BufferedImage myImage) throws IOException
	{	
		String cmdLn = "convert -quality " + (String)properties.get("jpeg_qual");
		cmdLn = cmdLn + " " + (String)properties.get("im_options") + " -geometry";		
		int height= myImage.getHeight();
		int width = myImage.getWidth();
		double ratio = 0;
		debug("Original file " + sourceFile.getPath() + " is " + width + "x" + height); 
		
		if(thumbUse.equals("ht"))
		{	
			ratio =height / newSize;
		}
		else if(thumbUse.equals("wd"))
		{
			ratio =width / newSize;
		}
		else
		{	
			ratio = Math.max(height, width) / newSize;
		}
 		ratio =  Math.max(ratio, 1.0);
		int newWidth = (int)(width / ratio);
	        int newHeight = (int)(height / ratio);
		debug("New file " + destinationFile.getPath() + " is " + newWidth + "x" + newHeight); 
		boolean wroteImage = false;
		if (!useShell)
		{
			Image scaledDownImage = myImage.getScaledInstance(newWidth, newHeight, Image.SCALE_SMOOTH);
			BufferedImage scaledDownBufferImage = new BufferedImage(newWidth, newHeight, BufferedImage.TYPE_INT_RGB);
			Graphics2D graphics2D = scaledDownBufferImage.createGraphics();
			graphics2D.drawImage(scaledDownImage, 0, 0, newWidth, newHeight, null);
			

			//String extension = sourceFile.getName();	
			//extension = extension.substring(extension.lastIndexOf(".")+1);
			String format = getFormatName(sourceFile);
			debug("Format of " + destinationFile.getPath() + " is " +format); 
			wroteImage = ImageIO.write(scaledDownBufferImage,format, destinationFile);
			
		}
		
		if(useShell || !wroteImage)
		{
			//try to write with shell command instead...
			String command = "";
			if(thumbMethod.equals("im"))
			{
				command= properties.get("impath") + "convert -quality " + properties.get("jpeg_qual");
				command = command + " " + properties.get("im_options") + " -geometry ";
				command = command + newWidth + "x" + newHeight + " \"" + sourceFile.getPath() + "\" \"" + destinationFile.getPath()+ "\"";
			}
			else
			{
				System.out.println("Sorry, importing with GD not supported...");
				System.exit(-30);
			}
				debug("Trying " + command + "...");
				Process process = Runtime.getRuntime().exec(command);
				int retval=0;
				try{
					retval = process.waitFor();	
				}	
				catch(InterruptedException ie)
				{
					ie.printStackTrace();
				}
				debug("exitcode: " + retval);
		}
		return (int)destinationFile.length();
	}
	private static String getFormatName(Object o) {
        try {

            ImageInputStream iis = ImageIO.createImageInputStream(o);
    

            Iterator iter = ImageIO.getImageReaders(iis);
            if (!iter.hasNext()) {

                return null;
            }
    

            ImageReader reader = (ImageReader)iter.next();
    

            iis.close();
    

            return reader.getFormatName();
        } catch (IOException e) {
        }

        return null;
    }

	public boolean isKnownFileType(String fileName)
	{
		return isImage(fileName) || isMovie(fileName) || isAudio(fileName) || isDocument(fileName);
	}
	
	public boolean isImage(String fileName)
	{
		return isType(fileName,"image");
	}

	public boolean isAudio(String fileName)
	{
		return isType(fileName,"audio");
	}

	public boolean isMovie(String fileName)
	{
		return isType(fileName,"movie");
	}

	public boolean isDocument(String fileName)
	{
		return isType(fileName,"document");
	}
	
	public boolean isType(String fileName, String type)
	{	
		String fetchedType = (String) fileTypes.get(fileName.substring(fileName.lastIndexOf(".")+1)) ;
		debug("Check if " + fileName + " (" + type +")" + " is of type " + fetchedType);
		if(fetchedType!= null)
		{

			if(fetchedType.equals(type))
			{
				debug(fileName + "(" + type +")" + " is of type " + fetchedType);
				return true;
			}
		}
			return false;
	}
	public static void debug(String s)
	{
		System.out.println(s);
	}	

}






class GCJDBCAdapter extends Thread
{

	Connection connection = null;
	private static GCJDBCAdapter adapter = null;
	
	
	private String username;
	private String password;
	private String database;
	
	private GCJDBCAdapter(String username, String password, String database)
	{
		this.username= username;
		this.password=password;
		this.database=database;
		initJDBCConnection(username, password, database);
		Runtime.getRuntime().addShutdownHook(this);
	}
	
	public static synchronized  GCJDBCAdapter getInstance() //returns null if we haven't connected to a database...
	{
		return adapter;
	}
	
	public static synchronized  GCJDBCAdapter getInstance(String username, String password, String database)
	{
		if (adapter == null)
		{
			adapter = new GCJDBCAdapter(username, password, database);
		}
		else{
			if(adapter.connection != null)
			{
				if(adapter.password.equals(password) && adapter.username.equals(username) && adapter.database.equals(database))
				{
				}
				else
				{
					try{ adapter.connection.close(); }
						catch(Exception e)
					{
						e.printStackTrace();
					}
					
					adapter = new GCJDBCAdapter(username, password, database);
				}
			}
		}
		return adapter;
	}
	
	private  synchronized  void initJDBCConnection(String username, String password, String database)
	{
		try {
    	 		 Class.forName("com.mysql.jdbc.Driver").newInstance();
    	 		 debug("Attempting to connect to mysql...");
	      		 connection = DriverManager.getConnection(database,username, password);
	      		 if(!connection.isClosed())
	      		 	debug("...Sucess! Autocommit is " + connection.getAutoCommit());
	      		 

		      if(!connection.isClosed())
			        debug("Successfully connected to " +
		          "MySQL server using TCP/IP...");
	
	    } catch(Exception e) {
		      System.err.println("Exception: " + e.getMessage());
		}
	}
	
	public synchronized  void executeUpdate(String SQL)
	{

	}
	
	public synchronized ResultSet  executeQuery(String SQL)
	{
		return null;
	}
	
	public synchronized void executeUpdate(PreparedStatement ps)
	{
		
	}
	
	public synchronized  ResultSet executeQuery(PreparedStatement ps)
	{
		return null;
	}
	
	
	public synchronized  PreparedStatement getPreparedStatement(String SQL)
	{
		try
		{
			return connection.prepareStatement(SQL);
		} catch(SQLException sqe)
		{
			sqe.printStackTrace();
		}
		return null;
	}
	
	public synchronized  static void commit() throws SQLException
	{
		adapter.connection.commit();
	}
	
	
	public synchronized  static void setAutoCommit(boolean autoCommit) throws SQLException
	{
		adapter.connection.setAutoCommit(autoCommit);
	}
	
	public synchronized void closeConnections()
	{
		try
		{
			debug("Closing connection....");
			connection.close();
		}
		catch(SQLException sqe)
		{
			sqe.printStackTrace();
		}
	}
	protected synchronized void finalize() throws Throwable
	{
		closeConnections();
		super.finalize();
	}
	public void run()
	{
 		closeConnections();
	}
	public static void debug(String s)
	{
		//System.out.println(s);
	}
}